@@ -14,7 +14,7 @@ use constant SKIP      => 2;
 #inherits Apache::Session
 #link Lemonldap::NG::Common::Apache::Session::SOAP protected globalStorage
 
-our $VERSION = '1.9.6';
+our $VERSION = '1.9.7';
 our ( %EXPORT_TAGS, @EXPORT_OK, @EXPORT );
 
 our $tsv = {};    # Hash ref containing thread-shared values, filled
@@ -81,10 +81,11 @@ sub forbidden {
     }
 
     # Log forbidding
+    my $vhost = $class->resolveAlias;
     Lemonldap::NG::Handler::Main::Logger->lmLog(
         "User "
           . $datas->{ $tsv->{whatToTrace} }
-          . " was forbidden to access to $uri",
+          . " was forbidden to access to $vhost$uri",
         "notice"
     );
     $class->updateStatus( 'REJECT', $datas->{ $tsv->{whatToTrace} } );
@@ -216,8 +217,11 @@ sub retrieveSession {
                 and $now - $datas->{_lastSeen} > $tsv->{timeoutActivity} )
           )
         {
-            Lemonldap::NG::Handler::Main::Logger->lmLog( "Session expired",
+            Lemonldap::NG::Handler::Main::Logger->lmLog( "Session $id expired",
                 'info' );
+
+            # Clean cached data
+            $datas = {};
             return 0;
         }
 
@@ -252,6 +256,44 @@ sub retrieveSession {
     }
 }
 
+## @rmethod protected hash getCDAInfos(id)
+# Tries to retrieve the CDA session, get infos and delete session
+# @return CDA session infos
+sub getCDAInfos {
+    my ( $class, $id ) = @_;
+    my $infos = {};
+
+    # Get the session
+    my $cdaSession = Lemonldap::NG::Common::Session->new(
+        {
+            storageModule        => $tsv->{sessionStorageModule},
+            storageModuleOptions => $tsv->{sessionStorageOptions},
+            cacheModule          => $tsv->{sessionCacheModule},
+            cacheModuleOptions   => $tsv->{sessionCacheOptions},
+            id                   => $id,
+            kind                 => "CDA",
+        }
+    );
+
+    unless ( $cdaSession->error ) {
+        Lemonldap::NG::Handler::Main::Logger->lmLog( "Get CDA session $id",
+            'debug' );
+
+        $infos->{cookie_value} = $cdaSession->data->{cookie_value};
+        $infos->{cookie_name}  = $cdaSession->data->{cookie_name};
+
+        $cdaSession->remove;
+    }
+    else {
+        Lemonldap::NG::Handler::Main::Logger->lmLog(
+            "CDA Session $id can't be retrieved", 'info' );
+        Lemonldap::NG::Handler::Main::Logger->lmLog( $cdaSession->error,
+            'info' );
+    }
+
+    return $infos;
+}
+
 # MAIN SUBROUTINE
 
 ## @rmethod int run(string ĉonf, string protection)
@@ -290,15 +332,26 @@ sub run {
     # Cross domain authentication
     my $uri = Lemonldap::NG::Handler::API->unparsed_uri;
     if (    $tsv->{cda}
-        and $uri =~ s/[\?&;]($tsv->{cookieName}(http)?=\w+)$//oi )
+        and $uri =~ s/[\?&;]$tsv->{cookieName}cda=(\w+)$//oi )
     {
-        my $str = $1;
-        Lemonldap::NG::Handler::Main::Logger->lmLog( 'CDA request', 'debug' );
+        my $cdaid = $1;
+        Lemonldap::NG::Handler::Main::Logger->lmLog(
+            "CDA request with id $cdaid", 'debug' );
+
+        my $cdaInfos = $class->getCDAInfos($cdaid);
+        unless ( $cdaInfos->{cookie_value} and $cdaInfos->{cookie_name} ) {
+            Lemonldap::NG::Handler::Main::Logger->lmLog(
+                "CDA request for id $cdaid is not valid", 'error' );
+            return FORBIDDEN;
+        }
+
         my $redirectUrl   = $class->_buildUrl($uri);
         my $redirectHttps = ( $redirectUrl =~ m/^https/ );
         Lemonldap::NG::Handler::API->set_header_out(
             'Location'   => $redirectUrl,
-            'Set-Cookie' => "$str; path=/"
+            'Set-Cookie' => $cdaInfos->{cookie_name} . "="
+              . $cdaInfos->{cookie_value}
+              . "; path=/"
               . ( $redirectHttps   ? "; secure"   : "" )
               . ( $tsv->{httpOnly} ? "; HttpOnly" : "" )
               . (
